@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 #-----------------------------------------------------------------------------------------------
-from allmydata.uri import CHKFileURI, NewDirectoryURI, LiteralFileURI
+from allmydata.uri import CHKFileURI, DirectoryURI, LiteralFileURI, is_literal_file_uri
 from allmydata.scripts.common_http import do_http as do_http_req
 from allmydata.util.hashutil import tagged_hash
 from allmydata.util.assertutil import precondition
@@ -335,7 +335,7 @@ class TahoeFuseFile(object):
                 self.fname = self.tfs.cache.tmp_file(os.urandom(20))
                 if self.fnode is None:
                     log('TFF: [%s] open() for write: no file node, creating new File %s' % (self.name, self.fname, ))
-                    self.fnode = File(0, 'URI:LIT:')
+                    self.fnode = File(0, LiteralFileURI.BASE_STRING)
                     self.fnode.tmp_fname = self.fname # XXX kill this
                     self.parent.add_child(self.name, self.fnode, {})
                 elif hasattr(self.fnode, 'tmp_fname'):
@@ -362,7 +362,7 @@ class TahoeFuseFile(object):
                     self.fname = self.fnode.tmp_fname
                     log('TFF: reopening(%s) for reading' % self.fname)
                 else:
-                    if uri.startswith("URI:LIT") or not self.tfs.async:
+                    if is_literal_file_uri(uri) or not self.tfs.async:
                         log('TFF: synchronously fetching file from cache for reading')
                         self.fname = self.tfs.cache.get_file(uri)
                     else:
@@ -906,7 +906,7 @@ def stat_to_dict(statobj, fields=None):
 
 class TStat(fuse.Stat):
     # in fuse 0.2, these are set by fuse.Stat.__init__
-    # in fuse 0.2-pre3 (hardy) they are not. badness unsues if they're missing
+    # in fuse 0.2-pre3 (hardy) they are not. badness ensues if they're missing
     st_mode  = None
     st_ino   = 0
     st_dev   = 0
@@ -1019,6 +1019,7 @@ class Directory(object):
     def get_uri(self):
         return self.rw_uri or self.ro_uri
 
+    # TODO: rename to 'is_writeable', or switch sense to 'is_readonly', for consistency with Tahoe code
     def writable(self):
         return self.rw_uri and self.rw_uri != self.ro_uri
 
@@ -1093,7 +1094,7 @@ class TFS(object):
 
         cachedir = os.path.expanduser(os.path.join(nodedir, '_cache'))
         self.cache = FileCache(nodeurl, cachedir)
-        ro_uri = NewDirectoryURI.init_from_string(self.root_uri).get_readonly()
+        ro_uri = DirectoryURI.init_from_string(self.root_uri).get_readonly()
         self.root = Directory(self, ro_uri, self.root_uri)
         self.root.maybe_refresh('<root>')
 
@@ -1183,7 +1184,7 @@ class TFS(object):
         url = self.compose_url("uri?t=mkdir")
         new_dir_cap = do_http('PUT', url)
 
-        ro_uri = NewDirectoryURI.init_from_string(new_dir_cap).get_readonly()
+        ro_uri = DirectoryURI.init_from_string(new_dir_cap).get_readonly()
         child = Directory(self, ro_uri, new_dir_cap)
         parent.add_child(name, child, {})
 
@@ -1237,7 +1238,7 @@ class FileCache(object):
 
     def get_file(self, uri):
         self.log('get_file(%s)' % (uri,))
-        if uri.startswith("URI:LIT"):
+        if is_literal_file_uri(uri):
             return self.get_literal(uri)
         else:
             return self.get_chk(uri, async=False)
@@ -1599,7 +1600,7 @@ def main(argv):
         root_name = 'uri_' + base32.b2a(tagged_hash('root_name', root_uri))[:12]
         # test the uri for structural validity:
         try:
-            NewDirectoryURI.init_from_string(root_uri)
+            DirectoryURI.init_from_string(root_uri)
         except:
             raise usage.error('root-uri must be a valid directory uri (not %r)' % (root_uri,))
     else:
