@@ -1,35 +1,35 @@
 import os.path, re, sys, fnmatch
 from twisted.python import usage
-from allmydata.scripts.common import BaseOptions, get_aliases
-from allmydata.util.encodingutil import argv_to_unicode
+from allmydata.scripts.common import BaseOptions, get_aliases, get_default_nodedir, DEFAULT_ALIAS
+from allmydata.util.encodingutil import argv_to_unicode, argv_to_abspath, quote_output
 
 NODEURL_RE=re.compile("http(s?)://([^:]*)(:([1-9][0-9]*))?")
 
-class VDriveOptions(BaseOptions, usage.Options):
+_default_nodedir = get_default_nodedir()
+
+class VDriveOptions(BaseOptions):
     optParameters = [
-        ["node-directory", "d", "~/.tahoe",
-         "Look here to find out which Tahoe node should be used for all "
-         "operations. The directory should either contain a full Tahoe node, "
-         "or a file named node.url which points to some other Tahoe node. "
-         "It should also contain a file named private/aliases which contains "
-         "the mapping from alias name to root dirnode URI."
-         ],
+        ["node-directory", "d", None,
+         "Specify which Tahoe node directory should be used. The directory "
+         "should either contain a full Tahoe node, or a file named node.url "
+         "that points to some other Tahoe node. It should also contain a file "
+         "named '" + os.path.join('private', 'aliases') + "' which contains the "
+         "mapping from alias name to root dirnode URI." + (
+            _default_nodedir and (" [default: " + quote_output(_default_nodedir) + "]") or "")],
         ["node-url", "u", None,
-         "URL of the tahoe node to use, a URL like \"http://127.0.0.1:3456\". "
+         "Specify the URL of the Tahoe gateway node, such as 'http://127.0.0.1:3456'. "
          "This overrides the URL found in the --node-directory ."],
         ["dir-cap", None, None,
-         "Which dirnode URI should be used as the 'tahoe' alias."]
+         "Specify which dirnode URI should be used as the 'tahoe' alias."]
         ]
 
     def postOptions(self):
-        # TODO: allow Unicode node-dir
-        # compute a node-url from the existing options, put in self['node-url']
         if self['node-directory']:
-            if sys.platform == 'win32' and self['node-directory'] == '~/.tahoe':
-                from allmydata.windows import registry
-                self['node-directory'] = registry.get_base_dir_path()
-            else:
-                self['node-directory'] = os.path.expanduser(self['node-directory'])
+            self['node-directory'] = argv_to_abspath(self['node-directory'])
+        else:
+            self['node-directory'] = _default_nodedir
+
+        # compute a node-url from the existing options, put in self['node-url']
         if self['node-url']:
             if (not isinstance(self['node-url'], basestring)
                 or not NODEURL_RE.match(self['node-url'])):
@@ -45,7 +45,7 @@ class VDriveOptions(BaseOptions, usage.Options):
 
         aliases = get_aliases(self['node-directory'])
         if self['dir-cap']:
-            aliases["tahoe"] = self['dir-cap']
+            aliases[DEFAULT_ALIAS] = self['dir-cap']
         self.aliases = aliases # maps alias name to dircap
 
 
@@ -57,19 +57,23 @@ class MakeDirectoryOptions(VDriveOptions):
 class AddAliasOptions(VDriveOptions):
     def parseArgs(self, alias, cap):
         self.alias = argv_to_unicode(alias)
+        if self.alias.endswith(u':'):
+            self.alias = self.alias[:-1]
         self.cap = cap
 
     def getSynopsis(self):
-        return "%s add-alias ALIAS DIRCAP" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s add-alias ALIAS[:] DIRCAP" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """Add a new alias for an existing directory."""
 
 class CreateAliasOptions(VDriveOptions):
     def parseArgs(self, alias):
         self.alias = argv_to_unicode(alias)
+        if self.alias.endswith(u':'):
+            self.alias = self.alias[:-1]
 
     def getSynopsis(self):
-        return "%s create-alias ALIAS" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s create-alias ALIAS[:]" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """Create a new directory and add an alias for it."""
 
@@ -78,11 +82,11 @@ class ListAliasOptions(VDriveOptions):
 
 class ListOptions(VDriveOptions):
     optFlags = [
-        ("long", "l", "Use long format: show file sizes, and timestamps"),
-        ("uri", "u", "Show file/directory URIs"),
-        ("readonly-uri", None, "Show readonly file/directory URIs"),
-        ("classify", "F", "Append '/' to directory names, and '*' to mutable"),
-        ("json", None, "Show the raw JSON output"),
+        ("long", "l", "Use long format: show file sizes, and timestamps."),
+        ("uri", "u", "Show file/directory URIs."),
+        ("readonly-uri", None, "Show read-only file/directory URIs."),
+        ("classify", "F", "Append '/' to directory names, and '*' to mutable."),
+        ("json", None, "Show the raw JSON output."),
         ]
     def parseArgs(self, where=""):
         self.where = argv_to_unicode(where)
@@ -131,7 +135,7 @@ class GetOptions(VDriveOptions):
             self.to_file = None
 
     def getSynopsis(self):
-        return "%s get REMOTE_FILE LOCAL_FILE" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s get REMOTE_FILE LOCAL_FILE" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """
     Retrieve a file from the grid and write it to the local filesystem. If
@@ -170,7 +174,7 @@ class PutOptions(VDriveOptions):
             self.from_file = None
 
     def getSynopsis(self):
-        return "%s put LOCAL_FILE REMOTE_FILE" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s put LOCAL_FILE REMOTE_FILE" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """
     Put a file into the grid, copying its contents from the local filesystem.
@@ -184,7 +188,7 @@ class PutOptions(VDriveOptions):
         t += """
 Examples:
  % cat FILE | tahoe put                # create unlinked file from stdin
- % cat FILE | tahoe -                  # same
+ % cat FILE | tahoe put -              # same
  % tahoe put bar                       # create unlinked file from local 'bar'
  % cat FILE | tahoe put - FOO          # create tahoe:FOO from stdin
  % tahoe put bar FOO                   # copy local 'bar' to tahoe:FOO
@@ -224,13 +228,13 @@ class CpOptions(VDriveOptions):
 
     You can also use a dircap as either FROM or TO target:
 
-    tahoe cp URI:DIR2-RO:j74uhg25nwdpjpacl6rkat2yhm:kav7ijeft5h7r7rxdp5bgtlt3viv32yabqajkrdykozia5544jqa/wiki.html ./   # copy Zooko's wiki page to a local file
+    tahoe cp URI:DIR2-RO:ixqhc4kdbjxc7o65xjnveoewym:5x6lwoxghrd5rxhwunzavft2qygfkt27oj3fbxlq4c6p45z5uneq/blog.html ./   # copy Zooko's wiki page to a local file
 
-    This command still has some limitations: symlinks, special files (device
-    nodes, named pipes), and non-ASCII filenames are not handled very well.
-    Arguments should probably not have trailing slashes. 'tahoe cp' does not
-    behave as much like /bin/cp as you would wish, especially with respect to
-    trailing slashes.
+    This command still has some limitations: symlinks and special files
+    (device nodes, named pipes) are not handled very well. Arguments should
+    probably not have trailing slashes. 'tahoe cp' does not behave as much
+    like /bin/cp as you would wish, especially with respect to trailing
+    slashes.
     """
 
 class RmOptions(VDriveOptions):
@@ -238,7 +242,11 @@ class RmOptions(VDriveOptions):
         self.where = argv_to_unicode(where)
 
     def getSynopsis(self):
-        return "%s rm REMOTE_FILE" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s rm REMOTE_FILE" % (os.path.basename(sys.argv[0]),)
+
+class UnlinkOptions(RmOptions):
+    def getSynopsis(self):
+        return "Usage:  %s unlink REMOTE_FILE" % (os.path.basename(sys.argv[0]),)
 
 class MvOptions(VDriveOptions):
     def parseArgs(self, frompath, topath):
@@ -246,7 +254,7 @@ class MvOptions(VDriveOptions):
         self.to_file = argv_to_unicode(topath)
 
     def getSynopsis(self):
-        return "%s mv FROM TO" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s mv FROM TO" % (os.path.basename(sys.argv[0]),)
     longdesc = """
     Use 'tahoe mv' to move files that are already on the grid elsewhere on
     the grid, e.g., 'tahoe mv alias:some_file alias:new_file'.
@@ -265,7 +273,29 @@ class LnOptions(VDriveOptions):
         self.to_file = argv_to_unicode(topath)
 
     def getSynopsis(self):
-        return "%s ln FROM TO" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s ln FROM_LINK TO_LINK" % (os.path.basename(sys.argv[0]),)
+
+    longdesc = """
+    Use 'tahoe ln' to duplicate a link (directory entry) already on the grid
+    to elsewhere on the grid. For example 'tahoe ln alias:some_file
+    alias:new_file'. causes 'alias:new_file' to point to the same object that
+    'alias:some_file' points to.
+
+    (The argument order is the same as Unix ln. To remember the order, you
+    can think of this command as copying a link, rather than copying a file
+    as 'tahoe cp' does. Then the argument order is consistent with that of
+    'tahoe cp'.)
+
+    When linking a remote file into a remote directory, you'll need to append
+    a '/' to the name of the remote directory, e.g. 'tahoe ln tahoe:file1
+    tahoe:dir/' (which is shorthand for 'tahoe ln tahoe:file1
+    tahoe:dir/file1'). If you forget the '/', e.g. 'tahoe ln tahoe:file1
+    tahoe:dir', the 'ln' command will refuse to overwrite the 'tahoe:dir'
+    directory, and will exit with an error.
+
+    Note that it is not possible to use this command to create links between
+    local and remote files.
+    """
 
 class BackupConfigurationError(Exception):
     pass
@@ -290,7 +320,7 @@ class BackupOptions(VDriveOptions):
         self.to_dir = argv_to_unicode(topath)
 
     def getSynopsis(Self):
-        return "%s backup FROM ALIAS:TO" % os.path.basename(sys.argv[0])
+        return "Usage:  %s backup FROM ALIAS:TO" % os.path.basename(sys.argv[0])
 
     def opt_exclude(self, pattern):
         """Ignore files matching a glob pattern. You may give multiple
@@ -303,10 +333,11 @@ class BackupOptions(VDriveOptions):
     def opt_exclude_from(self, filepath):
         """Ignore file matching glob patterns listed in file, one per
         line. The file is assumed to be in the argv encoding."""
+        abs_filepath = argv_to_abspath(filepath)
         try:
-            exclude_file = file(filepath)
+            exclude_file = file(abs_filepath)
         except:
-            raise BackupConfigurationError('Error opening exclude file %r.' % filepath)
+            raise BackupConfigurationError('Error opening exclude file %s.' % quote_output(abs_filepath))
         try:
             for line in exclude_file:
                 self.opt_exclude(line)
@@ -347,7 +378,7 @@ class WebopenOptions(VDriveOptions):
         self.where = argv_to_unicode(where)
 
     def getSynopsis(self):
-        return "%s webopen [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s webopen [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """Open a web browser to the contents of some file or
     directory on the grid. When run without arguments, open the Welcome
@@ -355,16 +386,16 @@ class WebopenOptions(VDriveOptions):
 
 class ManifestOptions(VDriveOptions):
     optFlags = [
-        ("storage-index", "s", "Only print storage index strings, not pathname+cap"),
-        ("verify-cap", None, "Only print verifycap, not pathname+cap"),
-        ("repair-cap", None, "Only print repaircap, not pathname+cap"),
-        ("raw", "r", "Display raw JSON data instead of parsed"),
+        ("storage-index", "s", "Only print storage index strings, not pathname+cap."),
+        ("verify-cap", None, "Only print verifycap, not pathname+cap."),
+        ("repair-cap", None, "Only print repaircap, not pathname+cap."),
+        ("raw", "r", "Display raw JSON data instead of parsed."),
         ]
     def parseArgs(self, where=''):
         self.where = argv_to_unicode(where)
 
     def getSynopsis(self):
-        return "%s manifest [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s manifest [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """Print a list of all files and directories reachable from
     the given starting point."""
@@ -377,23 +408,23 @@ class StatsOptions(VDriveOptions):
         self.where = argv_to_unicode(where)
 
     def getSynopsis(self):
-        return "%s stats [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s stats [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """Print statistics about of all files and directories
     reachable from the given starting point."""
 
 class CheckOptions(VDriveOptions):
     optFlags = [
-        ("raw", None, "Display raw JSON data instead of parsed"),
-        ("verify", None, "Verify all hashes, instead of merely querying share presence"),
-        ("repair", None, "Automatically repair any problems found"),
-        ("add-lease", None, "Add/renew lease on all shares"),
+        ("raw", None, "Display raw JSON data instead of parsed."),
+        ("verify", None, "Verify all hashes, instead of merely querying share presence."),
+        ("repair", None, "Automatically repair any problems found."),
+        ("add-lease", None, "Add/renew lease on all shares."),
         ]
     def parseArgs(self, where=''):
         self.where = argv_to_unicode(where)
 
     def getSynopsis(self):
-        return "%s check [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s check [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """
     Check a single file or directory: count how many shares are available and
@@ -402,17 +433,17 @@ class CheckOptions(VDriveOptions):
 
 class DeepCheckOptions(VDriveOptions):
     optFlags = [
-        ("raw", None, "Display raw JSON data instead of parsed"),
-        ("verify", None, "Verify all hashes, instead of merely querying share presence"),
-        ("repair", None, "Automatically repair any problems found"),
-        ("add-lease", None, "Add/renew lease on all shares"),
+        ("raw", None, "Display raw JSON data instead of parsed."),
+        ("verify", None, "Verify all hashes, instead of merely querying share presence."),
+        ("repair", None, "Automatically repair any problems found."),
+        ("add-lease", None, "Add/renew lease on all shares."),
         ("verbose", "v", "Be noisy about what is happening."),
         ]
     def parseArgs(self, where=''):
         self.where = argv_to_unicode(where)
 
     def getSynopsis(self):
-        return "%s deep-check [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
+        return "Usage:  %s deep-check [ALIAS:PATH]" % (os.path.basename(sys.argv[0]),)
 
     longdesc = """
     Check all files and directories reachable from the given starting point
@@ -427,11 +458,11 @@ subCommands = [
     ["ls", None, ListOptions, "List a directory."],
     ["get", None, GetOptions, "Retrieve a file from the grid."],
     ["put", None, PutOptions, "Upload a file into the grid."],
-    ["cp", None, CpOptions, "Copy one or more files."],
+    ["cp", None, CpOptions, "Copy one or more files or directories."],
     ["rm", None, RmOptions, "Unlink a file or directory on the grid."],
-    ["unlink", None, RmOptions, "Unlink a file or directory on the grid (same as rm)."],
+    ["unlink", None, UnlinkOptions, "Unlink a file or directory on the grid (same as rm)."],
     ["mv", None, MvOptions, "Move a file within the grid."],
-    ["ln", None, LnOptions, "Make an additional link to an existing file."],
+    ["ln", None, LnOptions, "Make an additional link to an existing file or directory."],
     ["backup", None, BackupOptions, "Make target dir look like local dir."],
     ["webopen", None, WebopenOptions, "Open a web browser to a grid file or directory."],
     ["manifest", None, ManifestOptions, "List all files/directories in a subtree."],
