@@ -4,11 +4,17 @@ import simplejson
 from twisted.protocols.basic import LineOnlyReceiver
 from allmydata.scripts.common import get_alias, DEFAULT_ALIAS, escape_path, \
                                      UnknownAliasError
-from allmydata.scripts.common_http import do_http
+from allmydata.scripts.common_http import do_http, format_http_error
+from allmydata.util.encodingutil import quote_output, quote_path
 
 class Checker:
     pass
 
+def _quote_serverid_index_share(serverid, storage_index, sharenum):
+    return "server %s, SI %s, shnum %r" % (quote_output(serverid, quotemarks=False),
+                                           quote_output(storage_index, quotemarks=False),
+                                           sharenum)
+
 def check(options):
     stdout = options.stdout
     stderr = options.stderr
@@ -19,7 +25,7 @@ def check(options):
     try:
         rootcap, path = get_alias(options.aliases, where, DEFAULT_ALIAS)
     except UnknownAliasError, e:
-        print >>stderr, "error: %s" % e.args[0]
+        e.display(stderr)
         return 1
     if path == '/':
         path = ''
@@ -37,7 +43,7 @@ def check(options):
 
     resp = do_http("POST", url)
     if resp.status != 200:
-        print >>stderr, "ERROR", resp.status, resp.reason, resp.read()
+        print >>stderr, format_http_error("ERROR", resp)
         return 1
     jdata = resp.read()
     if options.get("raw"):
@@ -54,38 +60,36 @@ def check(options):
             summary = "not healthy"
         stdout.write("Summary: %s\n" % summary)
         cr = data["pre-repair-results"]["results"]
-        stdout.write(" storage index: %s\n" % data["storage-index"])
-        stdout.write(" good-shares: %d (encoding is %d-of-%d)\n"
+        stdout.write(" storage index: %s\n" % quote_output(data["storage-index"], quotemarks=False))
+        stdout.write(" good-shares: %r (encoding is %r-of-%r)\n"
                      % (cr["count-shares-good"],
                         cr["count-shares-needed"],
                         cr["count-shares-expected"]))
-        stdout.write(" wrong-shares: %d\n" % cr["count-wrong-shares"])
+        stdout.write(" wrong-shares: %r\n" % cr["count-wrong-shares"])
         corrupt = cr["list-corrupt-shares"]
         if corrupt:
             stdout.write(" corrupt shares:\n")
             for (serverid, storage_index, sharenum) in corrupt:
-                stdout.write("  server %s, SI %s, shnum %d\n" %
-                             (serverid, storage_index, sharenum))
+                stdout.write("  %s\n" % _quote_serverid_index_share(serverid, storage_index, sharenum))
         if data["repair-attempted"]:
             if data["repair-successful"]:
                 stdout.write(" repair successful\n")
             else:
                 stdout.write(" repair failed\n")
     else:
-        stdout.write("Summary: %s\n" % data["summary"])
+        stdout.write("Summary: %s\n" % quote_output(data["summary"], quotemarks=False))
         cr = data["results"]
-        stdout.write(" storage index: %s\n" % data["storage-index"])
-        stdout.write(" good-shares: %d (encoding is %d-of-%d)\n"
+        stdout.write(" storage index: %s\n" % quote_output(data["storage-index"], quotemarks=False))
+        stdout.write(" good-shares: %r (encoding is %r-of-%r)\n"
                      % (cr["count-shares-good"],
                         cr["count-shares-needed"],
                         cr["count-shares-expected"]))
-        stdout.write(" wrong-shares: %d\n" % cr["count-wrong-shares"])
+        stdout.write(" wrong-shares: %r\n" % cr["count-wrong-shares"])
         corrupt = cr["list-corrupt-shares"]
         if corrupt:
             stdout.write(" corrupt shares:\n")
             for (serverid, storage_index, sharenum) in corrupt:
-                stdout.write("  server %s, SI %s, shnum %d\n" %
-                             (serverid, storage_index, sharenum))
+                stdout.write("  %s\n" % _quote_serverid_index_share(serverid, storage_index, sharenum))
     return 0
 
 
@@ -108,12 +112,12 @@ class DeepCheckOutput(LineOnlyReceiver):
 
     def lineReceived(self, line):
         if self.in_error:
-            print >>self.stderr, line
+            print >>self.stderr, quote_output(line, quotemarks=False)
             return
         if line.startswith("ERROR:"):
             self.in_error = True
             self.streamer.rc = 1
-            print >>self.stderr, line
+            print >>self.stderr, quote_output(line, quotemarks=False)
             return
 
         d = simplejson.loads(line)
@@ -135,17 +139,12 @@ class DeepCheckOutput(LineOnlyReceiver):
             if not path:
                 path = ["<root>"]
             summary = cr.get("summary", "Healthy (LIT)")
-            try:
-                print >>stdout, "%s: %s" % ("/".join(path), summary)
-            except UnicodeEncodeError:
-                print >>stdout, "%s: %s" % ("/".join([p.encode("utf-8")
-                                                      for p in path]),
-                                            summary)
+            print >>stdout, "%s: %s" % (quote_path(path), quote_output(summary, quotemarks=False))
+
         # always print out corrupt shares
         for shareloc in cr["results"].get("list-corrupt-shares", []):
             (serverid, storage_index, sharenum) = shareloc
-            print >>stdout, " corrupt: server %s, SI %s, shnum %d" % \
-                  (serverid, storage_index, sharenum)
+            print >>stdout, " corrupt: %s" % _quote_serverid_index_share(serverid, storage_index, sharenum)
 
     def done(self):
         if self.in_error:
@@ -174,12 +173,12 @@ class DeepCheckAndRepairOutput(LineOnlyReceiver):
 
     def lineReceived(self, line):
         if self.in_error:
-            print >>self.stderr, line
+            print >>self.stderr, quote_output(line, quotemarks=False)
             return
         if line.startswith("ERROR:"):
             self.in_error = True
             self.streamer.rc = 1
-            print >>self.stderr, line
+            print >>self.stderr, quote_output(line, quotemarks=False)
             return
 
         d = simplejson.loads(line)
@@ -221,18 +220,13 @@ class DeepCheckAndRepairOutput(LineOnlyReceiver):
                 summary = "healthy"
             else:
                 summary = "not healthy"
-            try:
-                print >>stdout, "%s: %s" % ("/".join(path), summary)
-            except UnicodeEncodeError:
-                print >>stdout, "%s: %s" % ("/".join([p.encode("utf-8")
-                                                      for p in path]),
-                                            summary)
+            print >>stdout, "%s: %s" % (quote_path(path), summary)
+
         # always print out corrupt shares
         prr = crr.get("pre-repair-results", {})
         for shareloc in prr.get("results", {}).get("list-corrupt-shares", []):
             (serverid, storage_index, sharenum) = shareloc
-            print >>stdout, " corrupt: server %s, SI %s, shnum %d" % \
-                  (serverid, storage_index, sharenum)
+            print >>stdout, " corrupt: %s" % _quote_serverid_index_share(serverid, storage_index, sharenum)
 
         # always print out repairs
         if crr["repair-attempted"]:
@@ -272,7 +266,7 @@ class DeepCheckStreamer(LineOnlyReceiver):
         try:
             rootcap, path = get_alias(options.aliases, where, DEFAULT_ALIAS)
         except UnknownAliasError, e:
-            print >>stderr, "error: %s" % e.args[0]
+            e.display(stderr)
             return 1
         if path == '/':
             path = ''
@@ -292,7 +286,7 @@ class DeepCheckStreamer(LineOnlyReceiver):
             url += "&add-lease=true"
         resp = do_http("POST", url)
         if resp.status not in (200, 302):
-            print >>stderr, "ERROR", resp.status, resp.reason, resp.read()
+            print >>stderr, format_http_error("ERROR", resp)
             return 1
 
         # use Twisted to split this into lines
