@@ -11,6 +11,7 @@ import allmydata # for __full_version__
 from allmydata import uri, monitor, client
 from allmydata.immutable import upload, encode
 from allmydata.interfaces import FileTooLargeError, UploadUnhappinessError
+from allmydata.util import log
 from allmydata.util.assertutil import precondition
 from allmydata.util.deferredutil import DeferredListShouldSucceed
 from allmydata.test.no_network import GridTestMixin
@@ -750,7 +751,7 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
         """
         I act like a normal upload, but before I send the results of
         Tahoe2PeerSelector to the Encoder, I break the first servers_to_break
-        PeerTrackers in the used_peers part of the return result.
+        PeerTrackers in the upload_servers part of the return result.
         """
         assert self.g, "I tried to find a grid at self.g, but failed"
         broker = self.g.clients[0].storage_broker
@@ -771,15 +772,15 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
         d = selector.get_shareholders(broker, sh, storage_index,
                                       share_size, block_size, num_segments,
                                       10, 3, 4)
-        def _have_shareholders((used_peers, already_peers)):
-            assert servers_to_break <= len(used_peers)
+        def _have_shareholders((upload_servers, already_peers)):
+            assert servers_to_break <= len(upload_servers)
             for index in xrange(servers_to_break):
-                server = list(used_peers)[index]
+                server = list(upload_servers)[index]
                 for share in server.buckets.keys():
                     server.buckets[share].abort()
             buckets = {}
             servermap = already_peers.copy()
-            for peer in used_peers:
+            for peer in upload_servers:
                 buckets.update(peer.buckets)
                 for bucket in peer.buckets:
                     servermap.setdefault(bucket, set()).add(peer.peerid)
@@ -798,9 +799,9 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
     def _add_server(self, server_number, readonly=False):
         assert self.g, "I tried to find a grid at self.g, but failed"
         ss = self.g.make_server(server_number, readonly)
+        log.msg("just created a server, number: %s => %s" % (server_number, ss,))
         self.g.add_server(server_number, ss)
 
-
     def _add_server_with_share(self, server_number, share_number=None,
                                readonly=False):
         self._add_server(server_number, readonly)
@@ -861,7 +862,6 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
         d.addCallback(_store_shares)
         return d
 
-
     def test_configure_parameters(self):
         self.basedir = self.mktemp()
         hooks = {0: self._set_up_nodes_extra_config}
@@ -1156,7 +1156,7 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
             self.failUnless(self._has_happy_share_distribution()))
         return d
 
-    def test_problem_layout_ticket1124(self):
+    def test_problem_layout_ticket_1124(self):
         self.basedir = self.mktemp()
         d = self._setup_and_upload(k=2, n=4)
 
@@ -1182,7 +1182,7 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
         d.addCallback(lambda ign:
             self.failUnless(self._has_happy_share_distribution()))
         return d
-    test_problem_layout_ticket1124.todo = "Fix this after 1.7.1 release."
+    test_problem_layout_ticket_1124.todo = "Fix this after 1.7.1 release."
 
     def test_happiness_with_some_readonly_peers(self):
         # Try the following layout
@@ -1342,7 +1342,7 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
 
 
     def test_merge_peers(self):
-        # merge_peers merges a list of used_peers and a dict of
+        # merge_peers merges a list of upload_servers and a dict of
         # shareid -> peerid mappings.
         shares = {
                     1 : set(["server1"]),
@@ -1351,7 +1351,7 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
                     4 : set(["server4", "server5"]),
                     5 : set(["server1", "server2"]),
                  }
-        # if not provided with a used_peers argument, it should just
+        # if not provided with a upload_servers argument, it should just
         # return the first argument unchanged.
         self.failUnlessEqual(shares, merge_peers(shares, set([])))
         class FakePeerTracker:
@@ -2086,3 +2086,11 @@ class EncodingParameters(GridTestMixin, unittest.TestCase, SetDEPMixin,
 #  upload with exactly 75 peers (shares_of_happiness)
 #  have a download fail
 #  cancel a download (need to implement more cancel stuff)
+
+# from test_encode:
+# NoNetworkGrid, upload part of ciphertext, kill server, continue upload
+# check with Kevan, they want to live in test_upload, existing tests might cover
+#     def test_lost_one_shareholder(self): # these are upload-side tests
+#     def test_lost_one_shareholder_early(self):
+#     def test_lost_many_shareholders(self):
+#     def test_lost_all_shareholders(self):
